/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
**
** Author: Ralph Meyer
**	   Oldenburg University
**	   Germany		
** 
** Module: asccond
**
** Purpose: 
**	Configuration facility
**
**	Module Prefix: CNF_
**
** Created:	06/1993
**
** Last Update:		$Author: meichel $
** Update Date:		$Date: 1998/12/23 13:18:40 $
** Source File:		$Source: /share/dicom/cvs-depot/dcmtk/imagectn/apps/cnfexnt.cc,v $
** CVS/RCS Revision:	$Revision: 1.4 $
** Status:		$State: Exp $
**
** CVS/RCS Log at end of file
**
*/



/*
 * includes
 */
#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#include <ctype.h>
#include <string.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
END_EXTERN_C

#include "cnfpriv.h"
#include "cnf.h"


/*
**	Function name : CNF_aeTitlesForPeer
**
**	Description : searches in the host table for all AE titles
**	known for peer hostName.  Creates an array of string pointers
**	containing the known AE titles.  The AE titles contained
**	in the array are privately owned by the config facility (you
**	may not free them).  You may free the array when no longer needed. 
**                    
**	Input Parameter: peer host name
**	Output Parameter: malloc'ed array of private string pointers.
**	Returns : number of entries in the malloced array.
**		  0 if no entries found.
*/

int
CNF_aeTitlesForPeer(const char *hostName, char *** aeTitleList)
{
    int n = 0;
    int i, j, k;
    const int chunkSize = 1;
    int maxAlloc = 0;
    char *hname;
    char *aetitle;
    int found;

    *aeTitleList = (char**)malloc(chunkSize*sizeof(char*));
    maxAlloc = chunkSize;

    /* collect up titles for peer, search in host table */
    for (i=0; i<CNF_HETable.noOfHostEntries; i++) {
	for (j=0; j<CNF_HETable.HostEntries[i].noOfPeers; j++) {
	    hname = CNF_HETable.HostEntries[i].Peers[j].HostName;
	    aetitle = CNF_HETable.HostEntries[i].Peers[j].ApplicationTitle;
#ifdef HAVE_PROTOTYPE_STRCASECMP
	    if (strcasecmp(hname, hostName) == 0) {  /* DNS is not case-sensitive */
#elif defined(HAVE_PROTOTYPE__STRICMP)
	    if (_stricmp(hname, hostName) == 0) {
#else
	    if (strcmp(hname, hostName) == 0) {  /* fallback if case insensitive compare is unavailable */
#endif
  		/* found an entry for peer host */
		/* make sure its not already in list */
		found = 0;
		for (k=0; !found && k<n; k++) {
		    found = (strcmp((*aeTitleList)[k], aetitle) == 0);
		}
		if (!found) {
		    if (n >= maxAlloc) {
		        *aeTitleList = (char**)realloc(*aeTitleList, 
			    (maxAlloc + chunkSize)*sizeof(char*));
		        maxAlloc += chunkSize;
		    }
		    (*aeTitleList)[n] = aetitle;

		    n++;
		}
	    }
	}
    }
    /* collect up titles for peer, search in AE table */
    for (i=0; i<CNF_Config.noOfAEEntries; i++) {
	for (j=0; j<CNF_Config.AEEntries[i].noOfPeers; j++) {
	    hname = CNF_Config.AEEntries[i].Peers[j].HostName;
	    aetitle = CNF_Config.AEEntries[i].Peers[j].ApplicationTitle;

#ifdef HAVE_PROTOTYPE_STRCASECMP
	    if (strcasecmp(hname, hostName) == 0) {  /* DNS is not case-sensitive */
#elif defined(HAVE_PROTOTYPE__STRICMP)
	    if (_stricmp(hname, hostName) == 0) {
#else
	    if (strcmp(hname, hostName) == 0) {  /* fallback if case insensitive compare is unavailable */
#endif
  		/* found an entry for peer host */
		/* make sure its not already in list */
		found = 0;
		for (k=0; !found && k<n; k++) {
		    found = (strcmp((*aeTitleList)[k], aetitle) == 0);
		}
		if (!found) {
		    if (n >= maxAlloc) {
		        *aeTitleList = (char**)realloc(*aeTitleList, 
			    (maxAlloc + chunkSize)*sizeof(char*));
		        maxAlloc += chunkSize;
		    }
		    (*aeTitleList)[n] = aetitle;

		    n++;
		}
	    }
	}
    }

    if (n == 0) {
	free(*aeTitleList);
	*aeTitleList = NULL;
    }
    return n;
}

/*
**	Function name : CNF_ctnTitles
**
**	Description : Creates an array of string pointers
**	containing the known AE titles for CTN storage areas.  
**	The AE titles contained in the array are privately owned 
**	by the config facility (you may not free them).  You may 
**	free the array when no longer needed. 
**                    
**	Output Parameter: malloc'ed array of private string pointers.
**	Returns : number of entries in the malloced array.
**		  0 if no entries exist.
*/

int 
CNF_ctnTitles(char *** ctnTitleList)
{
    int i;
    int n = 0;

    n = CNF_Config.noOfAEEntries;
    *ctnTitleList = (char**)malloc(n * sizeof(char*));

    for (i=0; i<n; i++) {
	(*ctnTitleList)[i] = CNF_Config.AEEntries[i].ApplicationTitle;
    }
    return n;
}


/*
**	Function name : CNF_aeTitlesForSymbolicName
**
**	Description : searches in the host table for all AE titles
**	known for a symbolic name.  Creates an array of string pointers
**	containing the known AE titles.  The AE titles contained
**	in the array are privately owned by the config facility (you
**	may not free them).  You may free the array when no longer needed. 
**                    
**	Input Parameter: symbolic name
**	Output Parameter: malloc'ed array of private string pointers.
**	Returns : number of entries in the malloced array.
**		  0 if no entries found.
*/

int 
CNF_aeTitlesForSymbolicName(const char *symbolicName, char *** aeTitleList)
{
    int i = 0;
    int j = 0;
    int n = 0;

    for (i=0; i<CNF_HETable.noOfHostEntries; i++) {
        if (strcmp(symbolicName, CNF_HETable.HostEntries[i].SymbolicName)==0) {
	    n = CNF_HETable.HostEntries[i].noOfPeers;
	    *aeTitleList = (char**)malloc(n * sizeof(char*));
	    for (j=0; j<n; j++) {
	        (*aeTitleList)[j] = 
		    CNF_HETable.HostEntries[i].Peers[j].ApplicationTitle;
	    }
	    return n;
        }
    }
    return 0;
    
}

const char*
CNF_vendorForPeerAETitle(const char *peerAETitle)
{
    int i = 0;
    int j = 0;
    
    for (i=0; i<CNF_VendorTable.noOfHostEntries; i++) {
	for (j=0; j<CNF_VendorTable.HostEntries[i].noOfPeers; j++) {
	    if (strcmp(peerAETitle, 
	        CNF_VendorTable.HostEntries[i].Peers[j].ApplicationTitle)==0) {
	        return CNF_VendorTable.HostEntries[i].SymbolicName;
	    }
        }
    }
    return NULL;
}

int CNF_countCtnTitles()
{
    return CNF_Config.noOfAEEntries;
}

/*
** CVS Log
** $Log: cnfexnt.cc,v $
** Revision 1.4  1998/12/23 13:18:40  meichel
** Corrected spelling of preprocessor symbols for strcasecmp and _stricmp
**
** Revision 1.3  1998/12/23 13:10:31  meichel
** Modified imagectn so that comparison of hostnames
**   (i.e. hostnames reported by DNS and the hostnames in the imagectn config file)
**   are case insensitive. Since DNS is case insensitive, this seems appropriate.
**
** Revision 1.2  1996/09/27 08:46:18  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
